package eu.etaxonomy.taxeditor;

import java.util.List;

import org.eclipse.e4.ui.model.application.ui.basic.MTrimElement;
import org.eclipse.e4.ui.model.application.ui.menu.MToolBarElement;
import org.eclipse.e4.ui.model.application.ui.menu.impl.ToolBarImpl;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;
import org.eclipse.ui.internal.WorkbenchWindow;

import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.handler.update.P2Util;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.util.ApplicationUtil;

/**
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public class ApplicationWorkbenchWindowAdvisor extends WorkbenchWindowAdvisor {

	public ApplicationWorkbenchWindowAdvisor(IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	@Override
    public ActionBarAdvisor createActionBarAdvisor(
			IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	@Override
    public void preWindowOpen() {
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(1280, 720));
		configurer.setShowCoolBar(true);
		configurer.setShowStatusLine(true);
		configurer.setShowPerspectiveBar(true);
		configurer.setTitle(ApplicationUtil.getTitle());
		configurer.setShowProgressIndicator(true);

		CdmDataSourceRepository.createDefaultH2DataSource();
	}

	@Override
    public void postWindowOpen() {
	    IWorkbenchWindow workbenchWindow = PlatformUI.getWorkbench().getActiveWorkbenchWindow();

	    List<MTrimElement> elements =  ((WorkbenchWindow) workbenchWindow).getTopTrim().getChildren();
	    for (MTrimElement item : elements) {
	        if (item instanceof ToolBarImpl){
	            ToolBarImpl toolBar = (ToolBarImpl)item;
	            List<MToolBarElement> children = toolBar.getChildren();
	            for (MToolBarElement element: children){
	                if (element.getElementId().equals("eu.etaxonomy.taxeditor.workbench.menu.toolbar.new")) {
	                    element.setVisible(false);
	                }
	            }
	        }

        }


		if(PreferencesUtil.shouldConnectAtStartUp()) {
            CdmStore.connect();
        }

		P2Util.setP2UpdateRepositories();

	}

	@Override
    public void postWindowClose() {
        P2Util.saveP2RepositoryPreferences();
    }
}

