// $Id$
/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.UIEvents;
import org.eclipse.e4.ui.workbench.lifecycle.PostContextCreate;
import org.eclipse.e4.ui.workbench.lifecycle.PreSave;
import org.eclipse.e4.ui.workbench.lifecycle.ProcessAdditions;
import org.eclipse.e4.ui.workbench.lifecycle.ProcessRemovals;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPerspectiveRegistry;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.osgi.service.event.Event;

import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @date 05.07.2017
 *
 */
public class LifeCycleManager implements IContextListener{

    private static final String NON_RESTORE = "nonRestore";

    private MApplication application;

    @Inject
    private EModelService modelService;

    @Inject
    public LifeCycleManager() {
        CdmStore.getContextManager().addContextListener(this);
    }

    @PostContextCreate
    private void postContextCreate() {
    }

    @ProcessAdditions
    private void processAdditions(MApplication application) {
        this.application = application;
        hideParts();

    }

    @ProcessRemovals
    private void processRemovals() {
    }

    @PreSave
    private void preSave(){
    }

    @Inject
    @Optional
    public void subscribeAppShutdownStarted(@UIEventTopic(UIEvents.UILifeCycle.APP_SHUTDOWN_STARTED) Event event,
            MApplication application) {
        IEclipseContext context = application.getContext();
        EPartService partService = context.get(EPartService.class);
        try {
            Collection<MPart> dirtyParts = partService.getDirtyParts();
            if(!dirtyParts.isEmpty()){
                partService.saveAll(true);
            }
        } catch (IllegalStateException e) {
            //FIXME E4
            //exception is ignored as this is a temporary fix until full e4 migration
        }
    }

    private void hideParts() {
        List<MPart> elements = modelService.findElements(application, null, MPart.class, Arrays.asList(NON_RESTORE));
        for (MPart part : elements) {
            MElementContainer<MUIElement> parent = part.getParent();
            if(parent!=null){
                if(parent.getSelectedElement()!=null
                        && parent.getSelectedElement().equals(part)){
                    parent.setSelectedElement(null);
                }
                parent.getChildren().remove(part);
            }
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextStop(IMemento memento, IProgressMonitor monitor) {
        hideParts();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextStart(IMemento memento, IProgressMonitor monitor) {
        PreferencesUtil.setDefaults();
        PreferencesUtil.updateDBPreferences();

        hideParts();
        if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowChecklistPerspective.getKey())){
            IWorkbench workbench = PlatformUI.getWorkbench();
            IWorkbenchWindow window = workbench
                    .getActiveWorkbenchWindow();

            IPerspectiveRegistry registry = workbench.getPerspectiveRegistry();
            IWorkbenchPage page = window.getActivePage();
            page.setPerspective(registry.findPerspectiveWithId("eu.etaxonomy.taxeditor.perspective.checklistperspective"));
        }else{
            IWorkbench workbench = PlatformUI.getWorkbench();
            IWorkbenchWindow window = workbench
                    .getActiveWorkbenchWindow();

            IPerspectiveRegistry registry = workbench.getPerspectiveRegistry();
            IWorkbenchPage page = window.getActivePage();
            page.setPerspective(registry.findPerspectiveWithId("eu.etaxonomy.taxeditor.application.perspective.taxonomic"));
        }
        PreferencesUtil.checkNomenclaturalCode();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextRefresh(IProgressMonitor monitor) {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
    }

}
